/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package GUI;

import Entities.Alap;
import Entities.Feltet;
import Logic.Logic;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 *
 * @author puzsaai
 */
public class GUI extends JFrame{
    private Logic logic;
    
    //Táblázatok
    private PizzaTable pizzaTable;
    private FeltetTable feltetTable;
    private JPanel panel1;
    private JPanel panel2;
    
    //Feltét manipuláló GUI elemek
    private AddFeltetAction addFeltetAction;
    private DeleteFeltetAction deleteFeltetAction;
    private JComboBox combobox;
    
    //Pizza ára
    private JLabel priceLabel;
    
    public GUI(){
        setSize(1000, 600);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        
        logic = Logic.getInstance();
        
        setGUI();
    }
    
    //Táblázatok felállítása
    private void setGUI(){
        setLayout(new FlowLayout());
        
        //Pizza-alap táblázat
        pizzaTable = new PizzaTable(this);
        panel1  = new JPanel();
        panel1.add( new JScrollPane(pizzaTable.getTable()) );
        
        //Feltétek táblázata
        feltetTable = new FeltetTable(null);//Nincs kiválasztott pizza-alap
        panel2  = new JPanel();
        panel2.add( new JScrollPane(feltetTable.getTable()) );
        
        add(panel1);
        add(panel2);
        
        //Hozzáadás/törlés gombok
        addFeltetAction = new AddFeltetAction();
        deleteFeltetAction = new DeleteFeltetAction();
        
        JButton addButton = new JButton(addFeltetAction);
        JButton delButton = new JButton(deleteFeltetAction);
        
        //Feltét választó doboz
        combobox = new JComboBox();
        for(Feltet f : logic.getFeltets()){
            combobox.addItem(f.getName());
        }
        
        add(combobox);
        add(addButton);
        add(delButton);
        
        priceLabel = new JLabel("Ár: ");
        add(priceLabel);
    }
    
    //Ha kijelölünk egy elemet a pizza-alap táblában, akkor frissíteni kell a 
    //hozzá tartozó pizza-feltet táblát, és a pizza árát
    public void refreshFeltetTable(){
        panel2.remove(0);//Levesszük a scrollPane-t
        
        //Majd hozzáadjuk az újat, egy másik alap feltétjeivel
        Alap a = pizzaTable.getSelectedAlap();
        feltetTable = new FeltetTable(a);
        panel2.add( new JScrollPane(feltetTable.getTable()) );
        
        //Pizza árának frissítése
        remove(priceLabel);
        priceLabel = new JLabel("Ár: " + a.getPriceWithFeltets());
        add(priceLabel);
        
        //Újrarajzolás
        validate();
        repaint();
    }
    
    private class AddFeltetAction extends AbstractAction{
        
        public AddFeltetAction(){
            super("Hozzáadás");
        }
        
        @Override
        public void actionPerformed(ActionEvent ae) {
            //Kijelölt elemeket lekérjük a táblázatokból
            Alap alap = pizzaTable.getSelectedAlap();
            if(alap != null){//Volt kijelölt pizza-alap
                
                //Lekérjük a combobox-ban kijelölt elemet
                String feltet = (String)combobox.getSelectedItem();

                logic.addFeltetToAlap(alap, feltet);//Logikai réteg módosítja az adatázist

                //Újra létrehozzuk a feltét táblázatot
                //Ez kicsit pazarlónak tűnik, de így érthetőbb a kód!
                refreshFeltetTable();
            }
        }
    }
    
    private class DeleteFeltetAction extends AbstractAction{
        
        public DeleteFeltetAction(){
            super("Törlés");
        }
        
        @Override
        public void actionPerformed(ActionEvent ae) {
            //Kijelölt elemeket lekérjük a táblázatokból
            Feltet feltet = feltetTable.getSelectedFeltet();
            Alap alap = pizzaTable.getSelectedAlap();
            
            //Ha volt kijelölve 1-1 elem táblákban akkor
            if(feltet != null && alap != null){
                //módosítjuk az adatbázist
                logic.deleteFeltetFromAlap(alap, feltet);

                //Újra létrehozzuk a feltét táblázatot
                //Ez kicsit pazarlónak tűnik, de így érthetőbb a kód!
                refreshFeltetTable();
            }
        }
    }
}
